#!/bin/bash

set -euo pipefail

############################################################
# Default Values                                           #
############################################################

ntasks=1
mem="16G"
time="02:00:00"
partition="amd-hdr100"
outdir=""

############################################################
# Help                                                     #
############################################################
usage()
{
>&2 cat << EOF
Usage: $0 [ -h ] [ -o | --outdir ] [ -n | --ntasks ] [ -p | --partition] [ -t | --time ] [ -m | --mem ] gpfs_logdir"
EOF
exit 1
}

help()
{
>&2 cat << EOF
Submits an array job to convert parts of a GPFS log to parquet format
Syntax: $0 [ -h ] [ -o | --outdir ] [ -n | --ntasks ] [ -p | --partition] [ -t | --time ] [ -m | --mem ] gpfs_logdir

options:
    -h|--help       Print this Help.

Required:
    gpfs_log_dir    Directory containing GPFS log outputs

Path:
   -o|--outdir      Directory to save parquet files to

sbatch options:
   -n|--ntasks      Number of tasks for each array index (default: 1)
   -p|--partition   Partition to submit tasks to (default: amd-hdr100)
   -t|--time        Max walltime (default: 02:00:00)
   -m|--mem         Memory for each task (default: 16G)
EOF
exit 0
}

args=$(getopt -a -o ho:n:p:t:m: --long help,outdir:,ntasks:,partition:,time:,mem: -- "$@")
if [[ $? -gt 0 ]]; then
  usage
fi

eval set -- ${args}

while :
do
  case $1 in
    -h | --help)      help           ;;
    -o | --outdir)    outdir=$2      ; shift 2 ;;
    -n | --ntasks)    ntasks=$2      ; shift 2 ;;
    -p | --partition) partition=$2   ; shift 2 ;;
    -t | --time)      time=$2        ; shift 2 ;;
    -m | --mem)       mem=$2         ; shift 2 ;;
    --) shift; break ;;
    *) >&2 echo Unsupported option: $1
       usage ;;
  esac
done

if [[ $# -eq 0 ]]; then
  usage
fi

gpfs_logdir="$1"

# Ensure gpfs_logdir is set
if [[ -z "$gpfs_logdir" ]]; then
    echo "Error: gpfs_logdir is a required positional argument."
    exit 1
fi

# If outdir not set, set to ${gpfs_logdir}/parquet
if [[ -z "$outdir" ]]; then
    outdir="${gpfs_logdir}/parquet"
fi

nlogs=$(ls ${gpfs_logdir}/list-* | wc -l)

cmd="python3 convert-to-parquet.py -o ${outdir} -f \${log}"

>&2 cat << EOF
--------------------------------------------------------------------------------
output dir:          ${outdir}
GPFS logs:           ${gpfs_logdir}

ntasks:              ${ntasks}
partition:           ${partition}
time:                ${time}
mem:                 ${mem}

command:             ${cmd}
--------------------------------------------------------------------------------
EOF

mkdir -p out
mkdir -p err

############################################################
# Create Array Job Script                                  #
############################################################

{ cat | sbatch; } << EOF
#!/bin/bash
#
#SBATCH --job-name=parquet-list-%a
#SBATCH --ntasks=${ntasks}
#SBATCH --partition=${partition}
#SBATCH --time=${time}
#SBATCH --mem=${mem}
#SBATCH --output=out/%A_%a.out
#SBATCH --error=err/%A_%a.err
#SBATCH --array=1-${nlogs}

source /data/rc/gpfs-policy/venv/bin/activate

log=\$(ls ${gpfs_logdir}/list-* | awk "NR==\${SLURM_ARRAY_TASK_ID} { print \$1 }")

${cmd}
EOF

exit 0